/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @defgroup nrf_twi_master_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Example Application main file.
 *
 * This file contains the source code for a sample application using TWI.
 */

#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "nrf_drv_rtc.h"
#include "nrf_drv_clock.h"
#include "bsp.h"
#include "app_uart.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_twi.h"
#include "lm75b.h"
#include "mma7660.h"
#include "nrf_delay.h"

/*********************************************/
/************Coulomb Counter******************/

#include <stdbool.h>
#include "nrf.h"
#include "nrf_drv_gpiote.h"
#include "nrf_gpio.h"


#define PIN_IN_INT  19
#define PIN_IN_POL  20


double battery_mAh = 2050.0;
double battery_percent = 100.0;
double ah_quanta = 0.176067759;
double percent_quanta;
uint8_t Pola=0;


void Current_Measure(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
	Pola = nrf_gpio_pin_read(PIN_IN_POL);
	
	if(Pola)
	{
		battery_mAh = battery_mAh + ah_quanta;
		battery_percent = battery_percent + percent_quanta;
	}
	else
	{
		battery_mAh = battery_mAh - ah_quanta;
		battery_percent = battery_percent - percent_quanta;
	}
}




static void gpio_init(void)
{
    ret_code_t err_code;

    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);

	//Interrupt PIN Configuration
    nrf_drv_gpiote_in_config_t in_config1 = GPIOTE_CONFIG_IN_SENSE_TOGGLE(true);
    in_config1.pull = NRF_GPIO_PIN_PULLUP;

    err_code = nrf_drv_gpiote_in_init(PIN_IN_INT, &in_config1, Current_Measure);
    APP_ERROR_CHECK(err_code);

    nrf_drv_gpiote_in_event_enable(PIN_IN_INT, true);
	
}



/*********************************************/




/*********************************************/
/****************MPR121***********************/

#define MPR121_I2CADDR_DEFAULT1 0x5A
#define MPR121_SOFTRESET1 0x80

#define MPR121_TOUCHSTATUS_L1 0x00
#define MPR121_TOUCHSTATUS_H1 0x01
#define MPR121_FILTDATA_0L1  0x04
#define MPR121_FILTDATA_0H1  0x05
#define MPR121_BASELINE_01   0x1E
#define MPR121_MHDR1         0x2B
#define MPR121_NHDR1         0x2C
#define MPR121_NCLR1         0x2D
#define MPR121_FDLR1         0x2E
#define MPR121_MHDF1         0x2F
#define MPR121_NHDF1         0x30
#define MPR121_NCLF1         0x31
#define MPR121_FDLF1         0x32
#define MPR121_NHDT1         0x33
#define MPR121_NCLT1         0x34
#define MPR121_FDLT1         0x35

#define MPR121_TOUCHTH_01    0x41
#define MPR121_RELEASETH_01     0x42
#define MPR121_DEBOUNCE1 0x5B
#define MPR121_CONFIG11 0x5C
#define MPR121_CONFIG21 0x5D
#define MPR121_CHARGECURR_01 0x5F
#define MPR121_CHARGETIME_11 0x6C
#define MPR121_ECR1 0x5E
#define MPR121_AUTOCONFIG01 0x7B
#define MPR121_AUTOCONFIG11 0x7C
#define MPR121_UPLIMIT1   0x7D
#define MPR121_LOWLIMIT1  0x7E
#define MPR121_TARGETLIMIT1  0x7F

#define MPR121_GPIODIR1  0x76
#define MPR121_GPIOEN1  0x77
#define MPR121_GPIOSET1  0x78
#define MPR121_GPIOCLR1  0x79
#define MPR121_GPIOTOGGLE1  0x7A



#define Temp_Addr1 0x80
#define Temp_Soft_Rst1 0xFE
#define Temp_UReg1 0xE7


uint8_t const Temp_Addr = Temp_Addr1;
uint8_t const Temp_Soft_Rst = Temp_Soft_Rst1;
uint8_t const Temp_UReg = Temp_UReg1;

static uint8_t const Sft[] = {Temp_Soft_Rst1};




uint8_t const  MPR121_I2CADDR_DEFAULT = MPR121_I2CADDR_DEFAULT1;
uint8_t const  MPR121_SOFTRESET = MPR121_SOFTRESET1;

uint8_t const  MPR121_TOUCHSTATUS_L = 0x00;
uint8_t const  MPR121_TOUCHSTATUS_H = 0x01;
uint8_t const  MPR121_FILTDATA_0L  =0x04;
uint8_t const  MPR121_FILTDATA_0H = 0x05;
uint8_t const  MPR121_BASELINE_0 =  0x1E;
uint8_t const  MPR121_MHDR      =   0x2B;
uint8_t const  MPR121_NHDR      =   0x2C;
uint8_t const  MPR121_NCLR      =   0x2D;
uint8_t const  MPR121_FDLR      =   0x2E;
uint8_t const  MPR121_MHDF      =   0x2F;
uint8_t const  MPR121_NHDF      =   0x30;
uint8_t const  MPR121_NCLF      =   0x31;
uint8_t const  MPR121_FDLF      =   0x32;
uint8_t const  MPR121_NHDT      =   0x33;
uint8_t const  MPR121_NCLT      =   0x34;
uint8_t const  MPR121_FDLT      =   0x35;

uint8_t const  MPR121_TOUCHTH_0   = 0x41;
uint8_t const  MPR121_RELEASETH_0 =   0x42;
uint8_t const  MPR121_DEBOUNCE =0x5B;
uint8_t const  MPR121_CONFIG1 =0x5C;
uint8_t const  MPR121_CONFIG2 =0x5D;
uint8_t const  MPR121_CHARGECURR_0 =0x5F;
uint8_t const  MPR121_CHARGETIME_1 =0x6C;
uint8_t const  MPR121_ECR =0x5E;
uint8_t const  MPR121_AUTOCONFIG0 =0x7B;
uint8_t const  MPR121_AUTOCONFIG1 =0x7C;
uint8_t const  MPR121_UPLIMIT  = 0x7D;
uint8_t const  MPR121_LOWLIMIT = 0x7E;
uint8_t const  MPR121_TARGETLIMIT = 0x7F;

uint8_t const  MPR121_GPIODIR = 0x76;
uint8_t const  MPR121_GPIOEN = 0x77;
uint8_t const  MPR121_GPIOSET = 0x78;
uint8_t const  MPR121_GPIOCLR = 0x79;
uint8_t const  MPR121_GPIOTOGGLE = 0x7A;


// Buffer for data read from MPR121.
#define MPR121_BUFFER_SIZE  10

static uint8_t MPR121_BUFFER[MPR121_BUFFER_SIZE];

static uint16_t MPR121_BUFFER1[2];

static app_twi_t m_app_twi = APP_TWI_INSTANCE(0);


// RESET MPR121
static uint8_t const Reset_Config[] = { MPR121_SOFTRESET1, 0x63 };

static uint8_t const MPR121_Detect[] = { MPR121_ECR1, 0x00 };

static uint8_t const Touch_Reg_0[] =   { 0x41, 0x0C };
static uint8_t const Touch_Reg_1[] =   { 0x43, 0x0C };
static uint8_t const Touch_Reg_2[] =   { 0x45, 0x0C };
static uint8_t const Touch_Reg_3[] =   { 0x47, 0xFF };
static uint8_t const Touch_Reg_4[] =   { 0x49, 0xFF };
static uint8_t const Touch_Reg_5[] =   { 0x4B, 0xFF };
static uint8_t const Touch_Reg_6[] =   { 0x4D, 0xFF };
static uint8_t const Touch_Reg_7[] =   { 0x4F, 0xFF };
static uint8_t const Touch_Reg_8[] =   { 0x51, 0xFF };
static uint8_t const Touch_Reg_9[] =   { 0x53, 0xFF };
static uint8_t const Touch_Reg_10[] =   { 0x55, 0xFF };
static uint8_t const Touch_Reg_11[] =   { 0x57, 0xFF };

static uint8_t const Release_Reg_0[] = { 0x42, 0x06 };
static uint8_t const Release_Reg_1[] = { 0x44, 0x06 };
static uint8_t const Release_Reg_2[] = { 0x46, 0x06 };
static uint8_t const Release_Reg_3[] = { 0x48, 0x06 };
static uint8_t const Release_Reg_4[] = { 0x4A, 0x06 };
static uint8_t const Release_Reg_5[] = { 0x4C, 0x06 };
static uint8_t const Release_Reg_6[] = { 0x4E, 0x06 };
static uint8_t const Release_Reg_7[] = { 0x50, 0x06 };
static uint8_t const Release_Reg_8[] = { 0x52, 0x06 };
static uint8_t const Release_Reg_9[] = { 0x54, 0x06 };
static uint8_t const Release_Reg_10[] = { 0x56, 0x06 };
static uint8_t const Release_Reg_11[] = { 0x58, 0x06 };


static uint8_t const MHDR_Reg[] = { MPR121_MHDR1, 0x01 };
static uint8_t const NHDR_Reg[] = { MPR121_NHDR1, 0x01 };
static uint8_t const NCLR_Reg[] = { MPR121_NCLR1, 0x0E };
static uint8_t const FDLR_Reg[] = { MPR121_FDLR1, 0x00 };
static uint8_t const MHDF_Reg[] = { MPR121_MHDF1, 0x01 };
static uint8_t const NHDF_Reg[] = { MPR121_NHDF1, 0x05 };
static uint8_t const NCLF_Reg[] = { MPR121_NCLF1, 0x01 };
static uint8_t const FDLF_Reg[] = { MPR121_FDLF1, 0x00 };
static uint8_t const NHDT_Reg[] = { MPR121_NHDT1, 0x00 };
static uint8_t const NCLT_Reg[] = { MPR121_NCLT1, 0x00 };
static uint8_t const FDLT_Reg[] = { MPR121_FDLT1, 0x00 };
static uint8_t const DEBOUNCE_Reg[] = { MPR121_DEBOUNCE1, 0x01 };
static uint8_t const CONFIG1_Reg[] = { MPR121_CONFIG11, 0x10 };
static uint8_t const CONFIG2_Reg[] = { MPR121_CONFIG21, 0x20 };
static uint8_t const ECR_Reg[] = { MPR121_ECR1, 0x8F };


app_twi_transfer_t const TEMP_RESET[] =
{

	//APP_TWI_WRITE(Temp_Addr1, Sft, sizeof(Sft), 0),
	
};


app_twi_transfer_t const MPR121_Reset[] =
{

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Reset_Config, sizeof(Reset_Config), 0),
	
};

app_twi_transfer_t const MPR121_Begin[] =
{

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, MPR121_Detect, sizeof(MPR121_Detect),0),

};


app_twi_transfer_t const MPR121_Init[]=
{

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_1, sizeof(Touch_Reg_0),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_1, sizeof(Release_Reg_0),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_2, sizeof(Touch_Reg_1),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_2, sizeof(Release_Reg_1),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_3, sizeof(Touch_Reg_2),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_3, sizeof(Release_Reg_2),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_1, sizeof(Touch_Reg_3),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_1, sizeof(Release_Reg_3),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_2, sizeof(Touch_Reg_4),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_2, sizeof(Release_Reg_4),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_3, sizeof(Touch_Reg_5),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_3, sizeof(Release_Reg_5),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_1, sizeof(Touch_Reg_6),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_1, sizeof(Release_Reg_6),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_2, sizeof(Touch_Reg_7),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_2, sizeof(Release_Reg_7),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_3, sizeof(Touch_Reg_8),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_3, sizeof(Release_Reg_8),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_1, sizeof(Touch_Reg_9),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_1, sizeof(Release_Reg_9),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_2, sizeof(Touch_Reg_10),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_2, sizeof(Release_Reg_10),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Touch_Reg_3, sizeof(Touch_Reg_11),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, Release_Reg_3, sizeof(Release_Reg_11),0),

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, MHDR_Reg, sizeof(MHDR_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, NHDR_Reg, sizeof(NHDR_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, NCLR_Reg, sizeof(NCLR_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, FDLR_Reg, sizeof(FDLR_Reg),0),

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, MHDF_Reg, sizeof(MHDF_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, NHDF_Reg, sizeof(NHDF_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, NCLF_Reg, sizeof(NCLF_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, FDLF_Reg, sizeof(FDLF_Reg),0),

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, NHDT_Reg, sizeof(NHDT_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, NCLT_Reg, sizeof(NCLT_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, FDLT_Reg, sizeof(FDLT_Reg),0),

	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, DEBOUNCE_Reg, sizeof(DEBOUNCE_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, CONFIG1_Reg, sizeof(CONFIG1_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, CONFIG2_Reg, sizeof(CONFIG2_Reg),0),
	APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, ECR_Reg, sizeof(ECR_Reg),0)

 };

uint8_t Detect = 0;


void MPR121_Check(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
        //printf("MPR121 - error: %d\n", (int)result);
		Detect = 0;
        return;
    }

	if (MPR121_BUFFER[0] != 0X24)
	{
		Detect = 0;
		return;
	}
	Detect = 1;
	return;

}

uint8_t const lm75b_temp_reg_addr11 = MPR121_CONFIG21;

extern uint8_t const lm75b_temp_reg_addr11;

static void MPR121_Valid(void)
{
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
		APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, &lm75b_temp_reg_addr11,1,APP_TWI_NO_STOP),
		APP_TWI_READ (MPR121_I2CADDR_DEFAULT1, &MPR121_BUFFER[0], 1, 0)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = MPR121_Check,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

	//uint8_t MPR121_RET = MPR121_BUFFER[0];

	//printf("MPR121_Return: %d\n", MPR121_RET);

	APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
}


void MPR121_Touch_Cal(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
        //printf("MPR121 - error: %d\n", (int)result);
		Detect = 0;
        return;
    }
	//printf("Current: %5.1f, Remaning: %5.1f",battery_mAh,battery_percent);
	printf("Result: %d,Temp: %d,Current: %5.1f, Remaning: %5.1f, Pol: %d \r\n", MPR121_BUFFER1[0],MPR121_BUFFER1[2],battery_mAh,battery_percent,Pola);
}



uint8_t const lm75b_temp_reg_addr12 = MPR121_TOUCHSTATUS_L1;
extern uint8_t const lm75b_temp_reg_addr12;

uint8_t const Temp_UReg_R = Temp_UReg1;
extern uint8_t const Temp_UReg_R;

static void MPR121_Touch_Detect(void)
{
	
	static app_twi_transfer_t const transfers[] =
    {
		APP_TWI_WRITE(MPR121_I2CADDR_DEFAULT1, &lm75b_temp_reg_addr12,1,APP_TWI_NO_STOP),
		APP_TWI_READ (MPR121_I2CADDR_DEFAULT1, &MPR121_BUFFER1[0], 2, 0)
		//APP_TWI_WRITE(Temp_Addr1, &Temp_UReg_R,1, APP_TWI_NO_STOP),
		//APP_TWI_READ (Temp_Addr1, &MPR121_BUFFER1[0], 2, 0)		
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = MPR121_Touch_Cal,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

	//uint8_t MPR121_RET = MPR121_BUFFER[0];

	//printf("MPR121_Return: %d\n", MPR121_RET);

	APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
	
}


/*********************************************/




#define MAX_PENDING_TRANSACTIONS    5

#define UART_TX_BUF_SIZE            256
#define UART_RX_BUF_SIZE            1

#define APP_TIMER_PRESCALER         0
#define APP_TIMER_OP_QUEUE_SIZE     2

// Pin number for indicating communication with sensors.
#ifdef BSP_LED_3
    #define READ_ALL_INDICATOR  BSP_LED_3
#else
    #error "Please choose an output pin"
#endif




static nrf_drv_rtc_t const m_rtc = NRF_DRV_RTC_INSTANCE(0);


// Buffer for data read from sensors.
#define BUFFER_SIZE  11
static uint8_t m_buffer[BUFFER_SIZE];

// Data structures needed for averaging of data read from sensors.
// [max 32, otherwise "int16_t" won't be sufficient to hold the sum
//  of temperature samples]
#define NUMBER_OF_SAMPLES  16
typedef struct
{
    int16_t temp;
    int16_t x;
    int16_t y;
    int16_t z;
} sum_t;
static sum_t m_sum = { 0, 0, 0, 0 };
typedef struct
{
    // [use bit fields to fit whole structure into one 32-bit word]
    int16_t temp : 11;
    int8_t  x    : 6;
    int8_t  y    : 6;
    int8_t  z    : 6;
} sample_t;
static sample_t m_samples[NUMBER_OF_SAMPLES] = { { 0, 0, 0, 0 } };
static uint8_t m_sample_idx = 0;

// Value previously read from MMA7660's Tilt register - used to detect change
// in orientation, shake signaling etc.
static uint8_t m_prev_tilt = 0;

#ifdef  __GNUC__
    // This is required if one wants to use floating-point values in 'printf'
    // (by default this feature is not linked together with newlib-nano).
    // Please note, however, that this adds about 13 kB code footprint...
    asm (".global _printf_float");
#endif


////////////////////////////////////////////////////////////////////////////////
// Reading of data from sensors - current temperature from LM75B and from
// MMA7660: X, Y, Z and tilt status.
#if (BUFFER_SIZE < 6)
    #error Buffer too small.
#endif
#define GET_ACC_VALUE(axis, reg_data) \
    do { \
        if (MMA7660_DATA_IS_VALID(reg_data)) \
        { \
            axis = MMA7660_GET_ACC(reg_data); \
        } \
    } while (0)
void read_all_cb(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
        //printf("read_all_cb - error: %d\r\n", (int)result);
        return;
    }

    sample_t * p_sample = &m_samples[m_sample_idx];
    m_sum.temp -= p_sample->temp;
    m_sum.x    -= p_sample->x;
    m_sum.y    -= p_sample->y;
    m_sum.z    -= p_sample->z;

    uint8_t temp_hi = m_buffer[0];
    uint8_t temp_lo = m_buffer[1];
    uint8_t x_out   = m_buffer[2];
    uint8_t y_out   = m_buffer[3];
    uint8_t z_out   = m_buffer[4];
    uint8_t tilt    = m_buffer[5];

    p_sample->temp = LM75B_GET_TEMPERATURE_VALUE(temp_hi, temp_lo);
    GET_ACC_VALUE(p_sample->x, x_out);
    GET_ACC_VALUE(p_sample->y, y_out);
    GET_ACC_VALUE(p_sample->z, z_out);
    if (!MMA7660_DATA_IS_VALID(tilt))
    {
        tilt = m_prev_tilt;
    }

    m_sum.temp += p_sample->temp;
    m_sum.x    += p_sample->x;
    m_sum.y    += p_sample->y;
    m_sum.z    += p_sample->z;

    ++m_sample_idx;
    if (m_sample_idx >= NUMBER_OF_SAMPLES)
    {
        m_sample_idx = 0;
    }

    // Show current average values every time sample index rolls over (for RTC
    // ticking at 32 Hz and 16 samples it will be every 500 ms) or when tilt
    // status changes.
    if (m_sample_idx == 0 || (m_prev_tilt && m_prev_tilt != tilt))
    {
        char const * orientation;
        switch (MMA7660_GET_ORIENTATION(tilt))
        {
            case MMA7660_ORIENTATION_LEFT:  orientation = "LEFT";  break;
            case MMA7660_ORIENTATION_RIGHT: orientation = "RIGHT"; break;
            case MMA7660_ORIENTATION_DOWN:  orientation = "DOWN";  break;
            case MMA7660_ORIENTATION_UP:    orientation = "UP";    break;
            default:                        orientation = "?";     break;
        }
        printf("Temp: %5.1f | X: %3d, Y: %3d, Z: %3d | %s%s%s\r\n",
            (m_sum.temp * 0.125) / NUMBER_OF_SAMPLES,
            m_sum.x / NUMBER_OF_SAMPLES,
            m_sum.y / NUMBER_OF_SAMPLES,
            m_sum.z / NUMBER_OF_SAMPLES,
            orientation,
            MMA7660_TAP_DETECTED(tilt)   ? " TAP"   : "",
            MMA7660_SHAKE_DETECTED(tilt) ? " SHAKE" : "");
        m_prev_tilt = tilt;
    }
}
static void read_all(void)
{
  /*  // Signal on LED that something is going on.
    nrf_gpio_pin_toggle(READ_ALL_INDICATOR);

    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        LM75B_READ_TEMP(&m_buffer[0])
        ,
        MMA7660_READ_XYZ_AND_TILT(&m_buffer[2])
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_all_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
    };

    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));*/
}


static void print_data(char const * name,
                       uint8_t const * p_buffer, uint8_t length)
{
   // printf("\r\n%s", name);

    do {
        //printf(" %02X", *p_buffer++);
    } while (--length);

    //printf("\r\n\r\n");
}


#if (BUFFER_SIZE < 7)
    #error Buffer too small.
#endif
static void read_lm75b_registers_cb(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
       // printf("read_lm75b_registers_cb - error: %d\r\n", (int)result);
        return;
    }

    print_data("LM75B:", m_buffer, 7);
}
static void read_lm75b_registers(void)
{
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        LM75B_READ(&lm75b_conf_reg_addr,  &m_buffer[0], 1),
        LM75B_READ(&lm75b_temp_reg_addr,  &m_buffer[1], 2),
        LM75B_READ(&lm75b_tos_reg_addr,   &m_buffer[3], 2),
        LM75B_READ(&lm75b_thyst_reg_addr, &m_buffer[5], 2)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_lm75b_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers)/sizeof(transfers[0])
    };

    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
}


#if (BUFFER_SIZE < MMA7660_NUMBER_OF_REGISTERS)
    #error Buffer too small.
#endif
static void read_mma7660_registers_cb(ret_code_t result, void * p_user_data)
{
    if (result != NRF_SUCCESS)
    {
        //printf("read_mma7660_registers_cb - error: %d\r\n", (int)result);
        return;
    }

    print_data("MMA7660:", m_buffer, MMA7660_NUMBER_OF_REGISTERS);
}
static void read_mma7660_registers(void)
{
    // [these structures have to be "static" - they cannot be placed on stack
    //  since the transaction is scheduled and these structures most likely
    //  will be referred after this function returns]
    static app_twi_transfer_t const transfers[] =
    {
        MMA7660_READ(&mma7660_xout_reg_addr,
            m_buffer, MMA7660_NUMBER_OF_REGISTERS)
    };
    static app_twi_transaction_t const transaction =
    {
        .callback            = read_mma7660_registers_cb,
        .p_user_data         = NULL,
        .p_transfers         = transfers,
        .number_of_transfers = sizeof(transfers)/sizeof(transfers[0])
    };

    APP_ERROR_CHECK(app_twi_schedule(&m_app_twi, &transaction));
}


////////////////////////////////////////////////////////////////////////////////
// Buttons handling (by means of BSP).
//
static void bsp_event_handler(bsp_event_t event)
{
    // Each time the button 1 or 4 is pushed we start a transaction reading
    // values of all registers from LM75B or MMA7660 respectively.
    switch (event)
    {
    case BSP_EVENT_KEY_0: // Button 1 pushed.
        read_lm75b_registers();
        break;

    case BSP_EVENT_KEY_3: // Button 4 pushed.
        read_mma7660_registers();
        break;

    default:
        break;
    }
}
static void bsp_config(void)
{
    uint32_t err_code;

    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, NULL);

    err_code = bsp_init(BSP_INIT_BUTTONS,
                        APP_TIMER_TICKS(100, APP_TIMER_PRESCALER),
                        bsp_event_handler);
    APP_ERROR_CHECK(err_code);
}


////////////////////////////////////////////////////////////////////////////////
// UART handling.
//
static void uart_event_handler(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
static void uart_config(void)
{
    uint32_t err_code;

    app_uart_comm_params_t const comm_params =
    {
        RX_PIN_NUMBER,
        TX_PIN_NUMBER,
        RTS_PIN_NUMBER,
        CTS_PIN_NUMBER,
        APP_UART_FLOW_CONTROL_ENABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud115200
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handler,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);
    APP_ERROR_CHECK(err_code);
}


////////////////////////////////////////////////////////////////////////////////
// TWI (with transaction manager) initialization.
//
static void twi_config(void)
{
    uint32_t err_code;

    nrf_drv_twi_config_t const config = {
       .scl                = ARDUINO_SCL_PIN,
       .sda                = ARDUINO_SDA_PIN,
       .frequency          = NRF_TWI_FREQ_100K,
       .interrupt_priority = APP_IRQ_PRIORITY_LOW
    };

    APP_TWI_INIT(&m_app_twi, &config, MAX_PENDING_TRANSACTIONS, err_code);
    APP_ERROR_CHECK(err_code);
}


////////////////////////////////////////////////////////////////////////////////
// RTC tick events generation.
//
static void rtc_handler(nrf_drv_rtc_int_type_t int_type)
{
    if (int_type == NRF_DRV_RTC_INT_TICK)
    {
        // On each RTC tick (their frequency is set in "nrf_drv_config.h")
        // we read data from our sensors.
        read_all();
		MPR121_Touch_Detect();
    }
}
static void rtc_config(void)
{
    uint32_t err_code;

    // Initialize RTC instance with default configuration.
    err_code = nrf_drv_rtc_init(&m_rtc, NULL, rtc_handler);
    APP_ERROR_CHECK(err_code);

    // Enable tick event and interrupt.
    nrf_drv_rtc_tick_enable(&m_rtc, true);

    // Power on RTC instance.
    nrf_drv_rtc_enable(&m_rtc);
}


static void lfclk_config(void)
{
    uint32_t err_code;

    err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);

    nrf_drv_clock_lfclk_request(NULL);
}


int main(void)
{
    LEDS_CONFIGURE(1U << READ_ALL_INDICATOR);
    LEDS_OFF(1U << READ_ALL_INDICATOR);

	nrf_gpio_cfg_input(PIN_IN_POL,NRF_GPIO_PIN_NOPULL);
	
	gpio_init();
	
    // Start internal LFCLK XTAL oscillator - it is needed by BSP to handle
    // buttons with the use of APP_TIMER and for "read_all" ticks generation
    // (by RTC).
    lfclk_config();

    bsp_config();

    uart_config();

    //printf("Stage_1\n");

    twi_config();
	
	percent_quanta = 1.0/(battery_mAh/1000.0*5859.0/100.0);
	
	APP_ERROR_CHECK(app_twi_perform(&m_app_twi, TEMP_RESET,1, NULL));
	
	// Initialize sensors.
	while(!Detect)
	{
		APP_ERROR_CHECK(app_twi_perform(&m_app_twi, MPR121_Reset,1, NULL));
		nrf_delay_ms(1);
		APP_ERROR_CHECK(app_twi_perform(&m_app_twi, MPR121_Begin,1, NULL));
		MPR121_Valid();				
	}
	printf("Started\r\n");
	APP_ERROR_CHECK(app_twi_perform(&m_app_twi, MPR121_Init,39, NULL));
    rtc_config();

    while (true)
    {
        __WFI();
    }
}


/** @} */

